@@ -1,5 +1,6 @@
 ;;; Directory Local Variables            -*- no-byte-compile: t -*-
 ;;; For more information see (info "(emacs) Directory Variables")
 
-((emacs-lisp-mode . ((eval . (display-fill-column-indicator-mode 1))
+((nil . ((project-vc-ignores . ("./README.org"))))
+ (emacs-lisp-mode . ((eval . (display-fill-column-indicator-mode 1))
                      (fill-column . 100))))
