@@ -131,7 +131,7 @@ Useful for when `save-excursion' does not preserve point."
   (if-let ((buffer (listen-queue-buffer queue)))
       (progn
         (pop-to-buffer buffer)
-        (listen-queue-goto-current))
+        (listen-queue-goto-current queue))
     (with-current-buffer
         (setf buffer (get-buffer-create (format "*Listen Queue: %s*" (listen-queue-name queue))))
       (let ((inhibit-read-only t))
@@ -139,7 +139,7 @@ Useful for when `save-excursion' does not preserve point."
         (setf listen-queue queue)
         (erase-buffer)
         (when (listen-queue-tracks listen-queue)
-          (make-vtable
+          (listen-make-vtable
            :columns
            (list (list :name "▶" :primary 'descend
                        :getter
@@ -236,7 +236,7 @@ Useful for when `save-excursion' does not preserve point."
                                 (call-interactively #'listen-library-from-queue))
                           "!" (lambda (_) (call-interactively #'listen-queue-shell-command)))))
         (listen-queue--annotate-buffer)
-        (listen-queue-goto-current)))
+        (listen-queue-goto-current queue)))
     ;; NOTE: We pop to the buffer outside of `with-current-buffer' so
     ;; `listen-queue--bookmark-handler' works correctly.
     (pop-to-buffer buffer)))
@@ -251,6 +251,7 @@ To be called in a queue's buffer."
          (inhibit-read-only t))
     (setf (map-elt (listen-queue-etc queue) :duration) duration)
     (vtable-end-of-table)
+    (delete-region (point) (point-max))
     (when duration
       (insert (format "Duration: %s" (listen-format-seconds duration))))
     (listen-queue--highlight-current)))
@@ -312,7 +313,7 @@ If BACKWARDP, move it backward."
                  (list track)
                  (seq-subseq (listen-queue-tracks queue) position)))
     (vtable-insert-object (vtable-current-table) track previous-track)
-    (vtable-update-object (vtable-current-table) previous-track previous-track)))
+    (listen-queue--vtable-update-object (vtable-current-table) previous-track previous-track)))
 
 (defun listen-queue--update-buffer (queue)
   "Update QUEUE's buffer, if any."
@@ -323,62 +324,20 @@ If BACKWARDP, move it backward."
       (when (vtable-current-table)
         (vtable-revert-command))
       (listen-queue--annotate-buffer))
-    (listen-queue-goto-current)))
+    (listen-queue-goto-current queue)))
 
-(defun listen-queue-update-track (track queue)
-  "Update TRACK in QUEUE.
-Reverts TRACK's metadata from the file and updates it in QUEUE,
-including QUEUE's buffer, if any."
-  ;; TODO: Use where appropriate.
-  (listen-queue-revert-track track)
-  (listen-queue-with-buffer queue
-    (listen-save-position
-      (goto-char (point-min))
-      (vtable-update-object (vtable-current-table) track track))))
-
-(declare-function listen-mode "listen")
-(declare-function listen-play "listen")
-;;;###autoload
-(cl-defun listen-queue-play (queue &optional (track (car (listen-queue-tracks queue))))
-  "Play QUEUE and optionally TRACK in it.
-Interactively, selected queue with completion; and with prefix,
-select track as well."
-  (interactive
-   (let* ((queue (listen-queue-complete))
-          (track (if current-prefix-arg
-                     (listen-queue-complete-track queue)
-                   (car (listen-queue-tracks queue)))))
-     (list queue track)))
-  (let ((player (listen-current-player)))
-    (listen-play player (listen-track-filename track))
-    (let ((previous-track (listen-queue-current queue)))
-      (setf (listen-queue-current queue) track
-            (map-elt (listen-player-etc player) :queue) queue)
-      (listen-queue-with-buffer queue
-        ;; HACK: Only update the vtable if its buffer is visible.
-        (when-let ((buffer-window (get-buffer-window (current-buffer))))
-          (with-selected-window buffer-window
-            (listen-save-position
-              (goto-char (point-min))
-              (ignore-errors
-                ;; HACK: Ignore errors, because if the window size has changed, the vtable's cache
-                ;; will miss and it will signal an error.
-                (when previous-track
-                  (listen-queue--vtable-update-object (vtable-current-table)
-                                                      previous-track previous-track))
-                (listen-queue--vtable-update-object (vtable-current-table) track track)))
-            (listen-queue--highlight-current))))))
-  (unless listen-mode
-    (listen-mode))
-  queue)
-
-(defun listen-queue-goto-current ()
+(defun listen-queue-goto-current (queue)
   "Jump to current track."
-  (interactive)
-  (when-let ((current-track (listen-queue-current listen-queue)))
-    ;; Ensure point is within the vtable.
-    (goto-char (point-min))
-    (vtable-goto-object current-track)))
+  (interactive (list (listen-queue-complete)))
+  (unless (listen-queue-buffer queue)
+    (listen-queue queue))
+  (listen-queue-with-buffer queue
+    (when-let ((current-track (listen-queue-current queue)))
+      ;; Ensure point is within the vtable.
+      (goto-char (point-min))
+      (vtable-goto-object current-track))
+    (unless (get-buffer-window (current-buffer))
+      (display-buffer (current-buffer)))))
 
 (defun listen-queue-complete-track (queue)
   "Return track selected from QUEUE with completion."
@@ -438,16 +397,26 @@ which see."
   (interactive (list (listen-queue-complete :prompt "Discard queue: ")))
   (cl-callf2 delete queue listen-queues))
 
+(defun listen-complete-files ()
+  ;; FIXME: Use this function in more places as appropriate.
+  "Return files selected with completion.
+In a Dired buffer, use `dired-get-marked-files'."
+  (declare-function dired-get-marked-files "dired")
+  (cl-case major-mode
+    (dired-mode (dired-get-marked-files))
+    (otherwise (let ((path (expand-file-name (read-file-name "Enqueue file/directory: "
+                                                             listen-directory nil t))))
+                 (if (file-directory-p path)
+                     (directory-files-recursively path ".")
+                   (list path))))))
+
 ;;;###autoload
 (cl-defun listen-queue-add-files (files queue)
-  "Add FILES to QUEUE."
+  "Add FILES to QUEUE.
+Completes files with `listen-complete-files', which see."
   (interactive
-   (let ((queue (listen-queue-complete :allow-new-p t))
-         (path (expand-file-name (read-file-name "Enqueue file/directory: " listen-directory nil t))))
-     (list (if (file-directory-p path)
-               (directory-files-recursively path ".")
-             (list path))
-           queue)))
+   (let ((queue (listen-queue-complete :allow-new-p t)))
+     (list (listen-complete-files) queue)))
   (cl-callf append (listen-queue-tracks queue) (listen-queue-tracks-for files))
   (listen-queue queue)
   queue)
@@ -459,10 +428,7 @@ the queue's buffer is updated, if any."
   (cl-callf append (listen-queue-tracks queue) tracks)
   ;; TODO: Consider updating the metadata of any duplicate tracks.
   (setf (listen-queue-tracks queue)
-        (cl-delete-duplicates (listen-queue-tracks queue)
-                              :key (lambda (track)
-                                     (expand-file-name (listen-track-filename track)))
-                              :test #'file-equal-p))
+        (listen-delete-dups (listen-queue-tracks queue) 'listen-track-equal))
   (listen-queue--update-buffer queue))
 
 (cl-defun listen-queue-add-from-playlist-file (filename queue)
@@ -552,9 +518,10 @@ with \"ffprobe\"."
     ;; If `listen-queue-track' (and thereby `listen-queue-tracks-for') returns nil for a track
     ;; (e.g. if its metadata can't be read), leave it alone (e.g. its metadata might have come from
     ;; by MPD).
+    ;; FIXME: Store metadata in its own slot and don't misuse etc slot.
+    (setf (listen-track-etc track) (listen-track-etc new-track)
+          (listen-track-metadata track) (listen-track-etc new-track))
     (dolist (slot '(artist title album number date genre etc))
-      ;; FIXME: Store metadata in its own slot and don't misuse etc slot.
-      (setf (listen-track-metadata track) (listen-track-etc new-track))
       (setf (cl-struct-slot-value 'listen-track slot track)
             (cl-struct-slot-value 'listen-track slot new-track)))))
 
@@ -627,12 +594,24 @@ tracks no longer backed by a file are removed."
       ;; the track metadata and refreshes the queue from disk while
       ;; the track is playing), in which case it won't be able to find
       ;; the track in the queue, so look again by comparing filenames.
-      (seq-elt (listen-queue-tracks queue)
-               (1+ (seq-position (listen-queue-tracks queue)
-                                 (listen-queue-current queue)
-                                 (lambda (a b)
-                                   (equal (listen-track-filename a)
-                                          (listen-track-filename b))))))))
+      (let ((current-track-position
+             (or (seq-position (listen-queue-tracks queue)
+                               (listen-queue-current queue)
+                               (lambda (a b)
+                                 (equal (expand-file-name (listen-track-filename a))
+                                        (expand-file-name (listen-track-filename b)))))
+                 (progn
+                   (display-warning 'listen-queue
+                                    (format-message "listen: Can't find track (%S) in queue (%S)"
+                                                    (listen-queue-current queue)
+                                                    (listen-queue-name queue))
+                                    :debug)
+                   (if-let ((track-number (map-elt (listen-queue-etc queue) :track-number)))
+                       ;; If the track was removed, the next track in the queue should have
+                       ;; taken its place, and we don't want to skip it, so subtract one.
+                       (1- track-number)
+                     (error "listen: Couldn't find track in queue, and track number is nil"))))))
+        (seq-elt (listen-queue-tracks queue) (1+ current-track-position)))))
 
 (declare-function listen-shell-command "listen")
 (defun listen-queue-shell-command (command filenames)
@@ -743,7 +722,7 @@ tracks in the queue unchanged)."
       (cl-labels ((get (slot)
                     (cons (capitalize (symbol-name slot))
                           (cl-struct-slot-value 'listen-track slot track))))
-        (make-vtable
+        (listen-make-vtable
          :columns
          (list (list :name "Key" :getter (lambda (row _table) (car row)))
                (list :name "Value" :getter (lambda (row _table) (cdr row))))
@@ -850,10 +829,11 @@ MAX-PROCESSES limits the number of parallel probing processes."
                                      "-show_entries" "format=duration"
                                      (expand-file-name (listen-track-filename track))))
                       (process (make-process
-                                :name "listen:ffprobe" :noquery t :type 'pipe :buffer (current-buffer)
-                                :sentinel sentinel :command (if listen-queue-nice-p
-                                                                (cons "nice" command)
-                                                              command))))
+                                :name "listen:ffprobe" :noquery t :connection-type 'pipe
+                                :buffer (current-buffer) :sentinel sentinel
+                                :command (if listen-queue-nice-p
+                                             (cons "nice" command)
+                                           command))))
                  process))))
          (probe-more ()
            (while (and tracks (length< processes max-processes))
@@ -914,7 +894,7 @@ Delay according to `listen-queue-delay-time-range', which see."
       (toggle-truncate-lines 1)
       (setq-local bookmark-make-record-function #'listen-queue-list--bookmark-make-record)
       (when listen-queues
-        (make-vtable
+        (listen-make-vtable
          :columns
          (list (list :name "▶" :primary 'descend
                      :getter (lambda (queue _table)
@@ -960,10 +940,41 @@ Delay according to `listen-queue-delay-time-range', which see."
     ;; `listen-queue--bookmark-handler' works correctly.
     (pop-to-buffer buffer)))
 
-;;;; Compatibility
+;;;; Vtable
+
+(require 'vtable)
+
+(cl-defmacro listen-with-vtable-at (position &rest body)
+  "FIXME: Docstring."
+  (declare (indent defun))
+  (let ((positionᵥ (gensym)))
+    `(let ((,positionᵥ ,position))
+       (save-excursion
+         (goto-char ,positionᵥ)
+         (cl-letf* (((symbol-function 'frame-terminal)
+                     (lambda (&optional _)
+                       listen-vtable-frame-terminal))
+                    ((symbol-function 'window-width)
+                     (lambda (&optional _ _)
+                       listen-vtable-window-width))
+                    (table (vtable-current-table))
+                    ((symbol-function 'vtable-current-table)
+                     (lambda ()
+                       table))
+                    ((symbol-function 'vtable--recompute-numerical)
+                     #'listen-queue--vtable--recompute-numerical))
+           ,@body)))))
+
+(defvar-local listen-vtable-frame-terminal nil)
+(defvar-local listen-vtable-window-width nil)
+
+(defun listen-make-vtable (&rest args)
+  (apply #'make-vtable args)
+  (setq-local listen-vtable-frame-terminal (frame-terminal)
+              listen-vtable-window-width (window-width)))
 
 (defalias 'listen-queue--vtable-update-object
-  (if (version<= emacs-version "29.2")
+  (if (version<= emacs-version "30")
       ;; See <https://debbugs.gnu.org/cgi/bugreport.cgi?bug=69664>.
       (lambda (table object old-object)
         "Replace OLD-OBJECT in TABLE with OBJECT."
@@ -1010,6 +1021,86 @@ Delay according to `listen-queue-delay-time-range', which see."
             (error "Can't find cached object in vtable"))))
     #'vtable-update-object))
 
+;;;;; Functions using these vtable workarounds
+
+(defun listen-queue-update-track (track queue)
+  "Update TRACK in QUEUE.
+Reverts TRACK's metadata from the file and updates it in QUEUE,
+including QUEUE's buffer, if any."
+  ;; TODO: Use where appropriate.
+  (listen-queue-revert-track track)
+  (listen-queue-with-buffer queue
+    (listen-save-position
+      (listen-with-vtable-at (point-min)
+        (listen-queue--vtable-update-object table track track)))))
+
+;;;###autoload
+(cl-defun listen-queue-play (queue &optional (track (car (listen-queue-tracks queue))))
+  "Play QUEUE and optionally TRACK in it.
+Interactively, selected queue with completion; and with prefix,
+select track as well."
+  (interactive
+   (let* ((queue (listen-queue-complete))
+          (track (if current-prefix-arg
+                     (listen-queue-complete-track queue)
+                   (car (listen-queue-tracks queue)))))
+     (list queue track)))
+  (declare-function listen-mode "listen")
+  (declare-function listen-play "listen")
+  (let ((player (listen-current-player)))
+    (listen-play player (listen-track-filename track))
+    ;; Remember queue position of track so if it gets removed, we can still go to the next track.
+    (setf (map-elt (listen-queue-etc queue) :track-number)
+          (seq-position (listen-queue-tracks queue) track))
+    (let ((previous-track (listen-queue-current queue)))
+      (setf (listen-queue-current queue) track
+            (map-elt (listen-player-etc player) :queue) queue)
+      (listen-queue-with-buffer queue
+        (listen-save-position
+          (listen-with-vtable-at (point-min)
+            (when previous-track
+              (listen-queue--vtable-update-object table previous-track previous-track))
+            (listen-queue--vtable-update-object table track track)))
+        (listen-queue--highlight-current))))
+  (unless listen-mode
+    (listen-mode))
+  queue)
+
+(defalias 'listen-queue--vtable--recompute-numerical
+  ;; TODO: Remove this when requiring Emacs 30+.
+  ;; See <https://debbugs.gnu.org/cgi/bugreport.cgi?bug=69927>.
+  (if (version< emacs-version "30.1")
+      (lambda (table line)
+        "Recompute numericalness of columns if necessary."
+        (let ((columns (vtable-columns table))
+              (recompute nil))
+          (seq-do-indexed
+           (lambda (elem index)
+             (when (and (vtable-column--numerical (elt columns index))
+                        (not (numberp (car elem))))
+               (setq recompute t)))
+           line)
+          (when recompute
+            (vtable--compute-columns table))))
+    ;; HACK: This is unique, one the likes of which I've never quite seen before.  This is to work
+    ;; around errors like "(cyclic-function-indirection vtable--recompute-numerical)" and "Symbol’s
+    ;; chain of function indirections contains a loop: vtable--recompute-numerical".  Because we
+    ;; also, in the macro `listen-with-vtable-at', dynamically rebind the function
+    ;; `vtable--recompute-numerical' with `cl-letf*', which normally creates a loop, we use
+    ;; `cl-letf' here also, to save a reference to the original function definition, which we make
+    ;; our own alias to.  Then when the expansion of `cl-letf*' in `listen-with-vtable-at' rebinds
+    ;; the function slot of `vtable--recompute-numerical', it binds it to the original function,
+    ;; rather than to the symbol (which would cause the cyclic indirection/loop).
+
+    ;; Now, you may think this is ugly or ridiculous, but it has a legitimate purpose: to provide a
+    ;; fix for users of older Emacs versions, while also being compatible with the Emacs version
+    ;; that has the fix included.  And how many other languages and platforms would even allow this?
+    ;; (Remember that `cl-letf' rebinds the symbol's function slot, so that while the macro's
+    ;; expansion is on the stack, anything else--in the whole system--that calls the rebound
+    ;; function calls our replacement for it--not just where we, ourselves, directly reference it.)
+    (cl-letf ((orig-fn (symbol-function 'vtable--recompute-numerical)))
+      orig-fn)))
+
 ;;;; Footer
 
 (provide 'listen-queue)
